@@ -539,10 +539,25 @@ impl ServiceDaemon {
 
             Command::StopBrowse(ty_domain) => match zc.queriers.remove_entry(&ty_domain) {
                 None => error!("StopBrowse: cannot find querier for {}", &ty_domain),
-                Some((_ty, sender)) => match sender.send(ServiceEvent::SearchStopped(ty_domain)) {
-                    Ok(()) => debug!("Sent SearchStopped to the listener"),
-                    Err(e) => error!("Failed to send SearchStopped: {}", e),
-                },
+                Some((ty, sender)) => {
+                    // Remove pending browse commands in the reruns.
+                    let mut i = 0;
+                    while i < zc.retransmissions.len() {
+                        if let Command::Browse(t, _, _) = &zc.retransmissions[i].command {
+                            if t == &ty {
+                                zc.retransmissions.remove(i);
+                                continue;
+                            }
+                        }
+                        i += 1;
+                    }
+
+                    // Notify the client.
+                    match sender.send(ServiceEvent::SearchStopped(ty_domain)) {
+                        Ok(()) => debug!("Sent SearchStopped to the listener"),
+                        Err(e) => error!("Failed to send SearchStopped: {}", e),
+                    }
+                }
             },
 
             Command::GetMetrics(resp_s) => match resp_s.send(zc.counters.clone()) {
@@ -1079,6 +1094,10 @@ impl Zeroconf {
         debug!("handle_query from {}", &addr);
         let mut out = DnsOutgoing::new(FLAGS_QR_RESPONSE | FLAGS_AA);
 
+        // Special meta-query "_services._dns-sd._udp.<Domain>".
+        // See https://datatracker.ietf.org/doc/html/rfc6763#section-9
+        const META_QUERY: &str = "_services._dns-sd._udp.local.";
+
         for question in msg.questions.iter() {
             debug!("question: {:?}", &question);
             let qtype = question.entry.ty;
@@ -1086,50 +1105,20 @@ impl Zeroconf {
             if qtype == TYPE_PTR {
                 for service in self.my_services.values() {
                     if question.entry.name == service.ty_domain {
+                        out.add_answer_with_additionals(&msg, service);
+                    } else if question.entry.name == META_QUERY {
                         let ptr_added = out.add_answer(
                             &msg,
                             Box::new(DnsPointer::new(
-                                &service.ty_domain,
+                                &question.entry.name,
                                 TYPE_PTR,
                                 CLASS_IN,
                                 service.other_ttl,
-                                service.fullname.clone(),
+                                service.ty_domain.clone(),
                             )),
                         );
-
                         if !ptr_added {
-                            debug!("answer was not added for question {:?}", &question);
-                            continue;
-                        }
-
-                        // Add recommended additional answers according to
-                        // https://tools.ietf.org/html/rfc6763#section-12.1.
-                        out.add_additional_answer(Box::new(DnsSrv::new(
-                            &service.fullname,
-                            CLASS_IN | CLASS_UNIQUE,
-                            service.host_ttl,
-                            service.priority,
-                            service.weight,
-                            service.port,
-                            service.server.clone(),
-                        )));
-
-                        out.add_additional_answer(Box::new(DnsTxt::new(
-                            &service.fullname,
-                            TYPE_TXT,
-                            CLASS_IN | CLASS_UNIQUE,
-                            service.host_ttl,
-                            service.generate_txt(),
-                        )));
-
-                        for address in &service.addresses {
-                            out.add_additional_answer(Box::new(DnsAddress::new(
-                                &service.server,
-                                TYPE_A,
-                                CLASS_IN | CLASS_UNIQUE,
-                                service.host_ttl,
-                                *address,
-                            )));
+                            debug!("answer was not added for meta-query {:?}", &question);
                         }
                     }
                 }
@@ -1397,6 +1386,9 @@ pub struct ServiceInfo {
 impl ServiceInfo {
     /// Creates a new service info.
     ///
+    /// `ty_domain` is the service type and the domain label, for example
+    /// "_my-service._udp.local.".
+    ///
     /// `my_name` is the instance name, without the service type suffix.
     /// `properties` are optional key/value pairs for the service.
     ///
@@ -1589,6 +1581,56 @@ impl DnsOutgoing {
         false
     }
 
+    /// Adds PTR answer and SRV, TXT, ADDR answers.
+    /// See https://tools.ietf.org/html/rfc6763#section-12.1
+    fn add_answer_with_additionals(&mut self, msg: &DnsIncoming, service: &ServiceInfo) {
+        let ptr_added = self.add_answer(
+            msg,
+            Box::new(DnsPointer::new(
+                &service.ty_domain,
+                TYPE_PTR,
+                CLASS_IN,
+                service.other_ttl,
+                service.fullname.clone(),
+            )),
+        );
+
+        if !ptr_added {
+            debug!("answer was not added for msg {:?}", msg);
+            return;
+        }
+
+        // Add recommended additional answers according to
+        // https://tools.ietf.org/html/rfc6763#section-12.1.
+        self.add_additional_answer(Box::new(DnsSrv::new(
+            &service.fullname,
+            CLASS_IN | CLASS_UNIQUE,
+            service.host_ttl,
+            service.priority,
+            service.weight,
+            service.port,
+            service.server.clone(),
+        )));
+
+        self.add_additional_answer(Box::new(DnsTxt::new(
+            &service.fullname,
+            TYPE_TXT,
+            CLASS_IN | CLASS_UNIQUE,
+            service.host_ttl,
+            service.generate_txt(),
+        )));
+
+        for address in &service.addresses {
+            self.add_additional_answer(Box::new(DnsAddress::new(
+                &service.server,
+                TYPE_A,
+                CLASS_IN | CLASS_UNIQUE,
+                service.host_ttl,
+                *address,
+            )));
+        }
+    }
+
     fn add_question(&mut self, name: &str, qtype: u16) {
         let q = DnsQuestion {
             entry: DnsEntry::new(name.to_string(), qtype, CLASS_IN),
@@ -2216,6 +2258,7 @@ fn check_service_name(fullname: &str) -> Result<()> {
     Ok(())
 }
 
+#[derive(Debug)]
 struct DnsIncoming {
     offset: usize,
     data: Vec<u8>,
