@@ -44,6 +44,8 @@ fn integration_success() {
     let resolve_count_clone = resolve_count.clone();
     let remove_count = Arc::new(Mutex::new(0));
     let remove_count_clone = remove_count.clone();
+    let stopped_count = Arc::new(Mutex::new(0));
+    let stopped_count_clone = stopped_count.clone();
 
     let browse_chan = d.browse(ty_domain).unwrap();
     std::thread::spawn(move || {
@@ -89,6 +91,8 @@ fn integration_success() {
                 }
                 ServiceEvent::SearchStopped(ty) => {
                     println!("Search stopped for {}", &ty);
+                    let mut num = stopped_count_clone.lock().unwrap();
+                    *num += 1;
                     break;
                 }
             }
@@ -125,6 +129,9 @@ fn integration_success() {
 
     sleep(Duration::from_secs(1));
 
+    let count = stopped_count.lock().unwrap();
+    assert_eq!(*count, 1);
+
     // Verify metrics.
     let metrics_receiver = d.get_metrics().unwrap();
     let metrics = metrics_receiver.recv().unwrap();
@@ -136,6 +143,47 @@ fn integration_success() {
     assert!(metrics["browse"] >= 2); // browse has been retransmitted.
     assert!(metrics["respond"] >= 2); // respond has been sent for every browse.
 
+    // Test the special meta-query of "_services._dns-sd._udp.local."
+    let service2_type = "_my-service2._udp.local.";
+    let service2_instance = "instance2";
+    let service2 = ServiceInfo::new(
+        service2_type,
+        service2_instance,
+        host_name,
+        host_ipv4,
+        port,
+        None,
+    );
+    d.register(service2)
+        .expect("Failed to register the 2nd service");
+
+    // Browse using the special meta-query.
+    let meta_query = "_services._dns-sd._udp.local.";
+    let browse_chan = d.browse(meta_query).unwrap();
+    let timeout = Duration::from_secs(2);
+
+    loop {
+        match browse_chan.recv_timeout(timeout) {
+            Ok(event) => match event {
+                ServiceEvent::ServiceFound(ty_domain, fullname) => {
+                    println!("Found a service of {}: {}", &ty_domain, &fullname);
+                    // Among all services found, should have our 2nd service.
+                    if fullname == service2_type {
+                        break;
+                    }
+                }
+                e => {
+                    println!("Received event {:?}", e);
+                    sleep(Duration::from_millis(100));
+                }
+            },
+            Err(e) => {
+                println!("browse error: {}", e);
+                assert!(false);
+            }
+        }
+    }
+
     // Shutdown
     d.shutdown().unwrap();
 }
